@@ -127,12 +127,20 @@ takes an incomplete SQL SELECT statement and massages it to return a DISTINCT re
 sub DistinctQuery {
     my $self = shift;
     my $statementref = shift;
-    my $table = shift;
-
-    # Wrapper select query in a subselect as Informix doesn't allow
-    # DISTINCT against CLOB/BLOB column types.
-    $$statementref = "SELECT * FROM $table main WHERE id IN ( SELECT DISTINCT main.id FROM $$statementref )";
+    my $sb = shift;
+    my $table = $sb->Table;
 
+    if ($sb->_OrderClause =~ /(?<!main)\./) {
+        # Don't know how to do ORDER BY when the DISTINCT is in a subquery
+        warn "Query will contain duplicate rows; don't how how to ORDER BY across DISTINCT";
+        $$statementref = "SELECT main.* FROM $$statementref";
+    } else {
+        # Wrapper select query in a subselect as Informix doesn't allow
+        # DISTINCT against CLOB/BLOB column types.
+        $$statementref = "SELECT * FROM $table main WHERE id IN ( SELECT DISTINCT main.id FROM $$statementref )";
+    }
+    $$statementref .= $sb->_GroupClause;
+    $$statementref .= $sb->_OrderClause;
 }
 
 
